; Copyright  2007 Atani Software, All rights reserved.
;
; You are free to use these macros for any purpose provided that
; you are releasing an addon for the game Sid Meier's Railroads!
;

!echo "Railroads! NSIS macros version 0.07 -- Copyright  2007 Atani Software"

; Include XML parser code
!include "XML.nsh"

; Logic Lib includes
!include "LogicLib.nsh"

;
; These defines are to be used for the GoodClass parameter
; of the CreateGoodIfNeeded macro.
;
!define GOODS_PASSENGER "<szClass>Passenger</szClass>"
!define GOODS_FREIGHT "<szClass>Freight</szClass>"
!define GOODS_FAST_FREIGHT "<szClass>FastFreight</szClass>"
!define GOODS_BULK_FREIGHT "<szClass>BulkFreight</szClass>"

!define GOOD_PREFIX_DEFAULT ""
!define GOOD_PREFIX_PASSENGER "<szNodePrefix>dummy_passenger_</szNodePrefix>"

!define GOOD_ANIM_CLASS_DEFAULT ""
!define GOOD_ANIM_CLASS_PASSENGER "<szAnimClass>CPassenger</szAnimClass>"

!define GOOD_ANIMS_DEFAULT ""
!define GOOD_ANIMS_PASSENGER "<Animations><szModel>passenger.kfm</szModel><szModel>passenger_man_B.kfm</szModel><szModel>passenger_woman_A.kfm</szModel><szModel>passenger_woman_B.kfm</szModel></Animations>"

!define GOOD_EVENT_DEFAULT ""
!define GOOD_EVENT_PASSENGER "<EventCodes><Idle1>0</Idle1><Idle2>6</Idle2><Fidget1>2</Fidget1><Fidget2>3</Fidget2><Fidget3>7</Fidget3><Fidget4>10</Fidget4><Fidget5>12</Fidget5><Walk>1</Walk><Run>8</Run></EventCodes>"

;
; These defines are to be used for any built-in Goods.
;
!define IND_GOOD_ARMS "Arms"
!define IND_GOOD_AUTOS "Automobiles"
!define IND_GOOD_BEER "Beer"
!define IND_GOOD_COAL "Coal"
!define IND_GOOD_CORNMEAL "Cornmeal"
!define IND_GOOD_FISH "Fish"
!define IND_GOOD_FOOD "Food"
!define IND_GOOD_GOLD "Gold"
!define IND_GOOD_GRAIN "Grain"
!define IND_GOOD_GRAPES "Grapes"
!define IND_GOOD_LIVESTOCK "Livestock"
!define IND_GOOD_MAIL "Mail"
!define IND_GOOD_MANUFACTURED_GOODS "Manufactured Goods"
!define IND_GOOD_MEDICINE "Medicine"
!define IND_GOOD_MILK "Milk"
!define IND_GOOD_NITRATES "Nitrates"
!define IND_GOOD_OIL "Oil"
!define IND_GOOD_ORE "Ore"
!define IND_GOOD_PAPER "Paper"
!define IND_GOOD_PASSENGER "Passenger"
!define IND_GOOD_STEEL "Steel"
!define IND_GOOD_SWINE "Swine"
!define IND_GOOD_TEXTILES "Textiles"
!define IND_GOOD_WINE "Wine"
!define IND_GOOD_WOOD "Wood"
!define IND_GOOD_WOOL "Wool"

;
; These defines should be used for the CreateIndustryIfNeeded
; macro for the IndustrySoundScape parameter.
;
!define IND_SOUND_DEFAULT ""
!define IND_SOUND_GRASSLAND "<szSoundScape>ASSS_TERRAIN_GRASSLAND</szSoundScape>"

;
; These defines should be used for the CreateTrainCarIfNeeded
; macro for the CarIcon parameter.
;
!define CAR_ICON_DEFAULT "icon_car_goods.dds"
!define CAR_ICON_CATTLE "icon_car_cattle.dds"
!define CAR_ICON_COAL "ICON_Car_Coal.dds"
!define CAR_ICON_FOOD "ICON_Car_Food.dds"
!define CAR_ICON_GRAIN "ICON_Car_Grain.dds"
!define CAR_ICON_LUMBER "ICON_Car_Lumber.dds"
!define CAR_ICON_MAIL "ICON_Car_Mail.dds"
!define CAR_ICON_OIL "ICON_Car_Oil.dds"
!define CAR_ICON_PAPER "ICON_Car_Paper.dds"
!define CAR_ICON_PASSENGER "ICON_Car_Passenger.dds"
!define CAR_ICON_STEEL "ICON_Car_Steel.dds"

;
; These defines should be used for the CreateTrainCarIfNeeded
; macro for the CarGroundClack and CarBridgeClack parameters.
;
!define CAR_CLACK_BRIDGE "AS3D_CLACK_PLANET_BRIDGE"
!define CAR_CLACK_GROUND "AS3D_CLACK_PLANET_GROUND"

;
; These defines should be used for CreateTrainIfNeeded
; macro for the TrainSharpAngleSound, TrainAverageAngleSound
; and TrainSlightAngleSound parameters.
;
!define TRAIN_SQUEAL_SHARP "AS3D_CURVE_SQUEAL"
!define TRAIN_SQUEAL_MEDIUM "AS3D_CURVE_SQUEAL_MEDIUM"
!define TRAIN_SQUEAL_LIGHT "AS3D_CURVE_SQUEAL_LIGHT"

;
; Loads a passed in XML file, using an XPath to search for a given node,
; if not found uses a second XPath to locate the proper insertion point
; for the given new node text.
;
; This is an internal macro which should not be used by mod authors in
; their NSIS scripts.
;
!macro _CreateXMLNodeIfNeeded XMLFileName ShortXMLFileName NodeName XPathNode GotoPathString NewNodeText
!insertmacro _CreateXMLNodeIfNeeded_PRE "${XMLFileName}" "${ShortXMLFileName}" "${NodeName}" "${XPathNode}" "${GotoPathString}" "${NewNodeText}"
!insertmacro _CreateXMLNodeIfNeeded_POST "${XMLFileName}" "${ShortXMLFileName}" "${NodeName}"
!macroend

;
; This is an internal macro which should not be used by mod authors in
; their NSIS scripts.
;
!macro _CreateXMLNodeIfNeeded_PRE XMLFileName ShortXMLFileName NodeName XPathNode GotoPathString NewNodeText
  DetailPrint "Checking for ${ShortXMLFileName} entry: ${NodeName}"
  ${xml::LoadFile} "${XMLFileName}" $0
  ${xml::RootElement} $0 $1
  ${xml::XPathNode} "${XPathNode}" $0
  ${if} $0 != 0
    DetailPrint "Creating ${NodeName}"
    ${xml::XPathNode} "${GotoPathString}" $0
    ${if} $0 != 0
      MessageBox MB_OK "Failed to traverse XPath in ${ShortXMLFileName}"
      Abort
    ${endif}
    !echo "_CreateXMLNodeIfNeeded: ${NewNodeText}"
    ${xml::CreateNode} "${NewNodeText}" $2
    ${if} $2 = 0
      MessageBox MB_OK "Failed to create ${NodeName} in ${ShortXMLFileName}"
      Abort
    ${endif}
    ${xml::InsertEndChild} $2 $0
    ${if} $0 != 0
      MessageBox MB_OK "Failed to insert ${NodeName} in ${ShortXMLFileName}"
      Abort
    ${endif}
!macroend

; This is an internal macro which should not be used by mod authors in
; their NSIS scripts.
;
!macro _CreateXMLNodeIfNeeded_POST XMLFileName ShortXMLFileName NodeName
    ${xml::SaveFile} "${XMLFileName}" $0
    ${if} $0 != 0
      MessageBox MB_OK "Failed to save ${XMLFileName}"
      Abort
    ${endif}
  ${else}
    DetailPrint "Found existing ${NodeName}"
  ${endif}
  ${xml::Unload}
!macroend

; Creates a RRT_Goods.xml entry if it does not exist
!macro CreateGoodIfNeeded \
       GoodName GoodClass GoodIcon GoodBitmapLarge GoodBitmapSmall \
       GoodBitmapTiny GoodBitmapCar GoodCarX GoodCarY GoodCarWidth \
       GoodCarHeight GoodBitmapCarEmpty GoodCarEmptyX GoodCarEmptyY \
       GoodCarEmptyWidth GoodCarEmptyHeight GoodColor GoodMinPrice \
       GoodMaxPrice GoodIndustryLeaderTitle GoodIndustryPlayerTitle \
       GoodAnimClass GoodAnimations GoodNodePrefix GoodEvents
!insertmacro _CreateXMLNodeIfNeeded_PRE \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Goods.xml" \
             "RRT_Goods.xml" \
             "${GoodName}" \
             "//RRTGoods/Good/szName[text()='${GoodName}']" \
             "//RRTGoods" \
             "<Good><szName>${GoodName}</szName>${GoodClass}<szGTMLIcon>${GoodIcon}</szGTMLIcon><BitmapLarge><szFile>${GoodBitmapLarge}</szFile></BitmapLarge><BitmapSmall><szFile>${GoodBitmapSmall}</szFile></BitmapSmall><BitmapTiny><szFile>${GoodBitmapTiny}</szFile></BitmapTiny><BitmapCar><szFile>${GoodBitmapCar}</szFile><uiX>${GoodCarX}</uiX><uiY>${GoodCarY}</uiY><uiWidth>${GoodCarWidth}</uiWidth><uiHeight>${GoodCarHeight}</uiHeight></BitmapCar><BitmapCarEmpty><szFile>${GoodBitmapCarEmpty}</szFile><uiX>${GoodCarEmptyX}</uiX><uiY>${GoodCarEmptyY}</uiY><uiWidth>${GoodCarEmptyWidth}</uiWidth><uiHeight>${GoodCarEmptyHeight}</uiHeight></BitmapCarEmpty><Color>${GoodColor}</Color><MinPrice>${GoodMinPrice}</MinPrice><MaxPrice>${GoodMaxPrice}</MaxPrice><szIndustryLeaderTitle>${GoodIndustryLeaderTitle}</szIndustryLeaderTitle><szIndustryPlayerTitle>${GoodIndustryPlayerTitle}</szIndustryPlayerTitle></Good>"
    ${if} "${GoodAnimClass}" != ""
       DetailPrint "Adding animation class for ${GoodName}"
       ${xml::CreateNode} "${GoodAnimClass}" $0
       ${xml::InsertEndChild} $0 $1
       ${xml::Parent} $0 $1
    ${endif}
    ${if} "${GoodAnimations}" != ""
       DetailPrint "Adding animations for ${GoodName}"
       ${xml::CreateNode} "${GoodAnimations}" $0
       ${xml::InsertEndChild} $0 $1
       ${xml::Parent} $0 $1
    ${endif}
    ${if} "${GoodNodePrefix}" != ""
       DetailPrint "Adding node prefix for ${GoodName}"
       ${xml::CreateNode} "${GoodNodePrefix}" $0
       ${xml::InsertEndChild} $0 $1
       ${xml::Parent} $0 $1
    ${endif}
    ${if} "${GoodEvents}" != ""
       DetailPrint "Adding events for ${GoodName}"
       ${xml::CreateNode} "${GoodEvents}" $0
       ${xml::InsertEndChild} $0 $1
       ${xml::Parent} $0 $1
    ${endif}
!insertmacro _CreateXMLNodeIfNeeded_POST \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Goods.xml" \
             "RRT_Goods.xml" \
             "${GoodName}"
!macroend

; Creates a RRT_Depots.xml entry if it does not exist
!macro CreateDepotIfNeeded \
       DepotName DepotModel DepotCreationKFM DepotInfluenceRadius \
       DepotIsAnnex DepotCost DepotFScale
!insertmacro _CreateXMLNodeIfNeeded \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Depots.xml" \
             "RRT_Depots.xml" \
             "${DepotName}" \
             "//RRTDepots/Depot/szName[text()='${DepotName}']" \
             "//RRTDepots" \
             "<Depot><szName>${DepotName}</szName><szModel>${DepotModel}</szModel><szCreateKFM>${DepotCreationKFM}</szCreateKFM><InfluenceRadius>${DepotInfluenceRadius}</InfluenceRadius><IsAnnex>${DepotIsAnnex}</IsAnnex><BuildCost>${DepotCost}</BuildCost><fScale>${DepotFScale}</fScale></Depot>"
!macroend

; Creates a RRT_Industries.xml entry if it does not exist
!macro CreateIndustryIfNeeded \
       IndustryName IndustryModel IndustrySecondModel IndustryGrowKFM \
       IndustryTerrainMap IndustryScale IndustryCoastal IndustryMountainous \
       IndustryInCity IndustryInForest IndustryCost IndustryAuctionImage \
       IndustryReportImage IndustryIcon IndustryResourceInput \
       IndustryResourceOutput IndustryResourceIORatio IndustryDisplayName \
       IndustrySoundScape
!insertmacro _CreateXMLNodeIfNeeded \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Industries.xml" \
             "RRT_Industries.xml" \
             "${IndustryName}" \
             "//RRTIndustries/Industries/RRTIndustry/szName[text()='${IndustryName}']" \
             "//RRTIndustries/Industries" \
             "<RRTIndustry><szName>${IndustryName}</szName><szModel>${IndustryModel}</szModel><szSecondModel>${IndustrySecondModel}</szSecondModel><szGrowKFM>${IndustryGrowKFM}</szGrowKFM><szTerrainmap>${IndustryTerrainMap}</szTerrainmap><fScale>${IndustryScale}</fScale><bIsCoastal>${IndustryCoastal}</bIsCoastal><bIsMountainous>${IndustryMountainous}</bIsMountainous><bInCity>${IndustryInCity}</bInCity><bInForests>${IndustryInForest}</bInForests><iCost>${IndustryCost}</iCost><Production><Resource><Input>${IndustryResourceInput}</Input><Output>${IndustryResourceOutput}</Output><InputOutputRatio>${IndustryResourceIORatio}</InputOutputRatio></Resource></Production>${IndustryDisplayName}<szAuctionImage>${IndustryAuctionImage}</szAuctionImage><szReportImage>${IndustryReportImage}</szReportImage><szIcon><szFile>${IndustryIcon}</szFile></szIcon>${IndustrySoundScape}</RRTIndustry>"
!macroend

; Creates a RRT_TrainCars.xml entry if it does not exist
!macro AddResourceToIndustry \
       IndustryName IndustryInput IndustryOutput IndustryRatio
  DetailPrint "Checking for ${IndustryName}/${IndustryInput}+${IndustryOutput} in RRT_Industries.xml"
  ${xml::LoadFile} "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Industries.xml" $0
  ${xml::RootElement} $0 $1
  ${xml::XPathNode} "//RRTIndustries/Industries/RRTIndustry/szName[text()='${IndustryName}']" $0
  ${if} $0 = 0
    ${xml::Parent} $0 $1
    ${xml::CreateNode} "<Resource><Input>${IndustryInput}</Input><Output>${IndustryOutput}</Output><InputOutputRatio>${IndustryRatio}</InputOutputRatio></Resource>" $2
    ${if} $2 = 0
      MessageBox MB_OK "Error Creating new node ${IndustryName}/${IndustryInput}+${IndustryOutput} in RRT_Industries.xml"
      Abort
    ${endif}
    ${xml::FirstChildElement} "Production" $0 $1
    ${if} $1 = -1
      DetailPrint "Adding Production and Resource nodes"
      ${xml::CreateNode} "<Production><Resource><Input>${IndustryInput}</Input><Output>${IndustryOutput}</Output><InputOutputRatio>${IndustryRatio}</InputOutputRatio></Resource></Production>" $2
      ${xml::InsertEndChild} $2 $0
    ${else}
      DetailPrint "Adding Resource node"
      ${xml::InsertEndChild} $2 $0
    ${endif}
    ${if} $0 != 0
      MessageBox MB_OK "Failed to insert ${IndustryName}/${IndustryInput}+${IndustryOutput} in RRT_Industries.xml"
      Abort
    ${endif}

    ${xml::SaveFile} "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Industries.xml" $0
    ${if} $0 != 0
      MessageBox MB_OK "Failed to save RRT_Industries.xml"
      Abort
    ${endif}
  ${else}
    MessageBox MB_OK "Failed to find Industry ${IndustryName}"
    Abort
  ${endif}
  ${xml::Unload}
!macroend

; Creates a RRT_TrainCars.xml entry if it does not exist
!macro CreateTrainCarIfNeeded \
       CarName CarGood CarModel CarDummyModel CarGroundClack CarBridgeClack CarRunLength CarIcon
!insertmacro _CreateXMLNodeIfNeeded \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_TrainCars.xml" \
             "RRT_TrainCars.xml" \
             "${CarName}" \
             "//RRTTrainCars/RRTGoodCars/TrainCar/szName[text()='${CarName}']" \
             "//RRTTrainCars/RRTGoodCars" \
             "<TrainCar><szName>${CarName}</szName><szGood>${CarGood}</szGood><szModel>${Carmodel}</szModel><szDummyModel>${CarDummyModel}</szDummyModel><szIcon>${CarIcon}</szIcon><szGroundClackSound>${CarGroundClack}</szGroundClackSound><szBridgeClackSound>${CarBridgeClack}</szBridgeClackSound><RunLength>${CarRunLength}</RunLength></TrainCar>"
!macroend

; Creates a RRT_TrainCars.xml entry if it does not exist
!macro CreateTenderCarIfNeeded \
       CarName CarModel CarDummyModel CarGroundClack CarBridgeClack CarRunLength
!insertmacro _CreateXMLNodeIfNeeded \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_TrainCars.xml" \
             "RRT_TrainCars.xml" \
             "${CarName}" \
             "//RRTTrainCars/RRTTenderCars/TenderCar/szName[text()='${CarName}']" \
             "//RRTTrainCars/RRTTenderCars" \
             "<TenderCar><szName>${CarName}</szName><szModel>${CarModel}</szModel><szDummyModel>${CarDummyModel}</szDummyModel><szGroundClackSound>${CarGroundClack}</szGroundClackSound><szBridgeClackSound>${CarBridgeClack}</szBridgeClackSound><RunLength>${CarRunLength}</RunLength></TenderCar>"
!macroend

; Creates a RRT_TrainCars.xml entry if it does not exist
!macro AssignTenderCarIfNeeded \
       TenderCarName CarTag
  DetailPrint "Checking for ${CarTag}/${TenderCarName} in RRT_Trains.xml"
  ${xml::LoadFile} "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Trains.xml" $0
  ${xml::RootElement} $0 $1
  ${xml::XPathNode} "//RRTTrains/Train/szName[text()='${CarTag}']" $0
  ${if} $0 = 0
    ${xml::Parent} $0 $1
    ${xml::CreateNode} "<szTenderCar>${TenderCarName}</szTenderCar>" $2
    ${if} $2 = 0
      MessageBox MB_OK "Failed to create ${CarTag}/${TenderCarName} in RRT_Trains.xml"
      Abort
    ${endif}
    ${xml::FirstChildElement} "szTenderCar" $0 $1
    ${if} $1 = -1
      DetailPrint "Creating ${CarTag}/${TenderCarName}"
      ${xml::InsertEndChild} $2 $0
    ${else}
      DetailPrint "Updating ${CarTag}/${TenderCarName}"
      ${xml::ReplaceNode} $2 $0
    ${endif}
    ${if} $0 != 0
      MessageBox MB_OK "Failed to insert ${CarTag}/${TenderCarName} in RRT_Trains.xml"
      Abort
    ${endif}

    ${xml::SaveFile} "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Trains.xml" $0
    ${if} $0 != 0
      MessageBox MB_OK "Failed to save RRT_Trains.xml"
      Abort
    ${endif}
  ${else}
    MessageBox MB_OK "Failed to find train ${CarTag}"
    Abort
  ${endif}
  ${xml::Unload}
!macroend

; Finds a RRT_TrainCars.xml entry, if it doesnt exist it calls Abort
!macro VerifyTrainExists \
       CarTag AbortMessage
  DetailPrint "Checking for ${CarTag} in RRT_Trains.xml"
  ${xml::LoadFile} "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Trains.xml" $0
  ${xml::RootElement} $0 $1
  ${xml::XPathNode} "//RRTTrains/Train/szName[text()='${CarTag}']" $0
  ${if} $0 = 0
    DetailPrint "Found ${CarTag}"
  ${else}
    MessageBox MB_OK "${AbortMessage}"
    Abort
  ${endif}
  ${xml::Unload}
!macroend

; Verifies if a specific FPK file exists
!macro VerifyFPKExists \
       FPKName AbortMessage
  !define ID ${__LINE__}
  DetailPrint "Checking for ${FPKName}"
  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\${FPKName}" VerifyFPKExists_${ID}
  MessageBox MB_OK "${AbortMessage}"
  Abort
VerifyFPKExists_${ID}:
  DetailPrint "Found ${FPKName}"
  !undef ID
!macroend


; Creates a RRT_Trains.xml entry if it does not exist
!macro CreateTrainIfNeeded \
       TrainTag TrainYear TrainMaker TrainModel TrainDummyModel TrainCarVersion TrainNewImage \
       TrainGroundClackSound TrainBridgeClackSound TrainMaxSpeed TrainSpeedZero TrainPower \
       TrainCost TrainMaintCost TrainIsDiesel TrainRunLength TrainHasEngineer TrainDefColor \
       TrainStartSoundCrossFade TrainSpeedStartWait TrainSpeedCrossFade TrainPitchChange \
       TrainStartSound TrainStopSound TrainSharpAngleSound TrainAverageAngleSound \
       TrainSlightAngleSound TrainSpeedSounds TrainBitmap
!insertmacro _CreateXMLNodeIfNeeded_PRE \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Trains.xml" \
             "RRT_Trains.xml" \
             "${TrainTag}" \
             "//RRTTrains/Train/szName[text()='${TrainTag}']" \
             "//RRTTrains]" \
             "<Train><szName>${TrainTag}</szName><Year>${TrainYear}</Year><szManufacturerName>${TrainMaker}</szManufacturerName><szModel>${TrainModel}</szModel><szDummyModel>${TrainDummyModel}</szDummyModel><szCar>${TrainCarVersion}</szCar><szNewTrainImage>${TrainNewImage}</szNewTrainImage><szGroundClackSound>${TrainGroundClackSound}</szGroundClackSound><szBridgeClackSound>${TrainBridgeClackSound}</szBridgeClackSound><MaxSpeed>${TrainMaxSpeed}</MaxSpeed><Speed0>${TrainSpeedZero}</Speed0><Power>${TrainPower}</Power><Cost>${TrainCost}</Cost><MaintenanceCost>${TrainMaintCost}</MaintenanceCost><Diesel>${TrainIsDiesel}</Diesel><RunLength>${TrainRunLength}</RunLength><HasEngineer>${TrainHasEngineer}</HasEngineer><rgbDefaultColor>${TrainDefColor}</rgbDefaultColor><fStartSoundCrossFade>${TrainStartSoundCrossFade}</fStartSoundCrossFade><fSpeedStartWait>${TrainSpeedStartWait}</fSpeedStartWait><fSpeedCrossFade>${TrainSpeedCrossFade}</fSpeedCrossFade><iPitchChange>${TrainPitchChange}</iPitchChange></Train>"

    ${if} "${TrainBitmap}" != ""
       DetailPrint "Adding TrainBitmap for ${TrainTag}"
       ${xml::CreateNode} "<BitmapIcon><szFile>${TrainBitmap}</szFile></BitmapIcon>" $0
       ${xml::InsertEndChild} $0 $1
       ${xml::Parent} $0 $1
    ${endif}
    ${if} "${TrainSpeedSounds}" != ""
       DetailPrint "Adding SpeedSounds for ${TrainTag}"
       ${xml::CreateNode} "${TrainSpeedSounds}" $0
       ${xml::InsertEndChild} $0 $1
       ${xml::Parent} $0 $1
    ${endif}
    ${if} "${TrainStartSound}" != ""
       DetailPrint "Adding Start Sound for ${TrainTag}"
       ${xml::CreateNode} "<szStartSound>${TrainStartSound}</szStartSound>" $0
       ${xml::InsertEndChild} $0 $1
       ${xml::Parent} $0 $1
    ${endif}
    ${if} "${TrainStopSound}" != ""
       DetailPrint "Adding Stop Sound for ${TrainTag}"
       ${xml::CreateNode} "<szStopSound>${TrainStopSound}</szStopSound>" $0
       ${xml::InsertEndChild} $0 $1
       ${xml::Parent} $0 $1
    ${endif}
    ${if} "${TrainSharpAngleSound}" != ""
       DetailPrint "Adding Sharp Angle Sound for ${TrainTag}"
       ${xml::CreateNode} "<szSharpAngleSound>${TrainSharpAngleSound}</szSharpAngleSound>" $0
       ${xml::InsertEndChild} $0 $1
       ${xml::Parent} $0 $1
    ${endif}
    ${if} "${TrainAverageAngleSound}" != ""
       DetailPrint "Adding Average Angle Sound for ${TrainTag}"
       ${xml::CreateNode} "<szAverageAngleSound>${TrainAverageAngleSound}</szAverageAngleSound>" $0
       ${xml::InsertEndChild} $0 $1
       ${xml::Parent} $0 $1
    ${endif}
    ${if} "${TrainSlightAngleSound}" != ""
       DetailPrint "Adding Slight Angle Sound for ${TrainTag}"
       ${xml::CreateNode} "<szSlightAngleSound>${TrainSlightAngleSound}</szSlightAngleSound>" $0
       ${xml::InsertEndChild} $0 $1
       ${xml::Parent} $0 $1
    ${endif}

!insertmacro _CreateXMLNodeIfNeeded_POST \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Trains.xml" \
             "RRT_Trains.xml" \
             "${TrainTag}"
!macroend

;
; This macro checks if the "global" XML files have been installed
; previously, if not installs them.
;
!macro CheckGlobalXMLs GlobalXMLSource
  !define ID ${__LINE__}
  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Goods.xml" GoodsXMLFound_${ID}
    File "${GlobalXMLSource}\RRT_Goods.xml"
GoodsXMLFound_${ID}:

  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Depots.xml" DepotsXMLFound_${ID}
    File "${GlobalXMLSource}\RRT_Depots.xml"
DepotsXMLFound_${ID}:

  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Industries.xml" IndustriesXMLFound_${ID}
    File "${GlobalXMLSource}\RRT_Industries.xml"
IndustriesXMLFound_${ID}:

  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_Trains.xml" TrainsXMLFound_${ID}
    File "${GlobalXMLSource}\RRT_Trains.xml"
TrainsXMLFound_${ID}:

  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\RRT_TrainCars.xml" TrainCarsXMLFound_${ID}
    File "${GlobalXMLSource}\RRT_TrainCars.xml"
TrainCarsXMLFound_${ID}:

  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_DEU.xml" Names_DEUXMLFound_${ID}
    File "${GlobalXMLSource}\Names_DEU.xml"
Names_DEUXMLFound_${ID}:

  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_ENU.xml" Names_ENUXMLFound_${ID}
    File "${GlobalXMLSource}\Names_ENU.xml"
Names_ENUXMLFound_${ID}:

  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_ESP.xml" Names_ESPXMLFound_${ID}
    File "${GlobalXMLSource}\Names_ESP.xml"
Names_ESPXMLFound_${ID}:

  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_FRA.xml" Names_FRAXMLFound_${ID}
    File "${GlobalXMLSource}\Names_FRA.xml"
Names_FRAXMLFound_${ID}:

  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_ITA.xml" Names_ITAXMLFound_${ID}
    File "${GlobalXMLSource}\Names_ITA.xml"
Names_ITAXMLFound_${ID}:

  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_NLD.xml" Names_NLDXMLFound_${ID}
    File "${GlobalXMLSource}\Names_NLD.xml"
Names_NLDXMLFound_${ID}:

  !undef ID
!macroend

; Creates a Names_XXXX.xml entry if it does not exist
!macro CreateGlobalNameIfNeeded \
       NameTag NameText NameGender
!insertmacro _CreateXMLNodeIfNeeded \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_ENU.xml" \
             "Names_ENU.xml" \
             "${NameTag}" \
             "//Names/RRTTrains/TextKey/Tag[text()='${NameTag}']" \
             "//Names/RRTTrains" \
             "<TextKey><Tag>${NameTag}</Tag><Text>${NameText}</Text><Gender>${NameGender}</Gender></TextKey>"
!insertmacro _CreateXMLNodeIfNeeded \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_DEU.xml" \
             "Names_DEU.xml" \
             "${NameTag}" \
             "//Names/RRTTrains/TextKey/Tag[text()='${NameTag}']" \
             "//Names/RRTTrains" \
             "<TextKey><Tag>${NameTag}</Tag><Text>${NameText}</Text><Gender>${NameGender}</Gender></TextKey>"
!insertmacro _CreateXMLNodeIfNeeded \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_ESP.xml" \
             "Names_ESP.xml" \
             "${NameTag}" \
             "//Names/RRTTrains/TextKey/Tag[text()='${NameTag}']" \
             "//Names/RRTTrains" \
             "<TextKey><Tag>${NameTag}</Tag><Text>${NameText}</Text><Gender>${NameGender}</Gender></TextKey>"
!insertmacro _CreateXMLNodeIfNeeded \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_FRA.xml" \
             "Names_FRA.xml" \
             "${NameTag}" \
             "//Names/RRTTrains/TextKey/Tag[text()='${NameTag}']" \
             "//Names/RRTTrains" \
             "<TextKey><Tag>${NameTag}</Tag><Text>${NameText}</Text><Gender>${NameGender}</Gender></TextKey>"
!insertmacro _CreateXMLNodeIfNeeded \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_ITA.xml" \
             "Names_ITA.xml" \
             "${NameTag}" \
             "//Names/RRTTrains/TextKey/Tag[text()='${NameTag}']" \
             "//Names/RRTTrains" \
             "<TextKey><Tag>${NameTag}</Tag><Text>${NameText}</Text><Gender>${NameGender}</Gender></TextKey>"
!insertmacro _CreateXMLNodeIfNeeded \
             "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\XML\Names_NLD.xml" \
             "Names_NLD.xml" \
             "${NameTag}" \
             "//Names/RRTTrains/TextKey/Tag[text()='${NameTag}']" \
             "//Names/RRTTrains" \
             "<TextKey><Tag>${NameTag}</Tag><Text>${NameText}</Text><Gender>${NameGender}</Gender></TextKey>"
!macroend


!macro GetSkinVersion \
	SKIN_NAME \
	SKIN_DIFF_NAME \
	SKIN_DIFF_NAME_LEN \
	MAX_VERSION
  !define ID ${__LINE__}
  DetailPrint "Searching for other ${SKIN_NAME} skins"
  DetailPrint "(this may take a few minutes...)"
  
  ${RecFindOpen} "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\" $R0 $R1
  ${RecFindFirst}
  
  IfFileExists "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets\$R0\$R1" "" find_next_file_${ID}
    
  StrLen $R2 $R1
  IntOp $R2 $R2 - 3
  StrCpy $R3 $R1 3 $R2
  StrCmp $R3 "fpk" fpk_file_${ID}
  StrCmp $R3 "dds" dds_file_${ID}
  goto find_next_file_${ID}
  
fpk_file_${ID}:
  DetailPrint "Parsing CustomAssets$R0\$R1"
  ${smr::SearchFPK} "$DOCUMENTS\My Games\Sid Meier's Railroads!\CustomAssets$R0\$R1" "${SKIN_DIFF_NAME}" $R5
  Pop $R6
  IntCmp $R5 0 find_next_file_${ID} find_next_file_${ID} add_more_${ID}
  
dds_file_${ID}:
  DetailPrint "Checking $R0\$R1"
  StrLen $R2 $R1
  IntOp $R2 $R2 - 3
  StrCpy $R6 $R1 $R2
  StrCpy $R7 $R6 ${SKIN_DIFF_NAME_LEN}
  StrCmp $R7 "${SKIN_DIFF_NAME}" add_one_${ID} find_next_file_${ID}
add_one_${ID}:
  IntOp ${MAX_VERSION} ${MAX_VERSION} + 1
  goto find_next_file_${ID}
  
add_more_${ID}:
  IntOp ${MAX_VERSION} ${MAX_VERSION} + $R5
  
find_next_file_${ID}:
  ${RecFindNext}
  ${RecFindClose}

!macroend

!define smr::SearchFPK `!insertmacro smr::SearchFPK`

!macro smr::SearchFPK _FILE _SEARCH _RESULT
  smr::_ParseFPK `${_FILE}` `${_SEARCH}`;
  Pop ${_RESULT}
!macroend

;
; Changelog
; 2007-03-19 -- Added industry sound scape support.
; 2007-03-15 -- Added more Car Icon defines,
;               support for installing train skins, version 0.06
; 2007-03-06 -- Fixed depot creation [thanks choobacca], version 0.05
; 2007-03-04 -- Fixed tender car association code, version 0.04
; 2007-03-04 -- Added support for adding trains and tender
;               cars, version 0.03
; 2007-03-03 -- Added support for animations, events, car icons
;               and node-prefixs, version 0.02
; 2007-03-01 -- general cleanup and releasing version 0.01
; 2007-02-28 -- initial version posted to hooked forums
;

